from matchescu.typing._data import Record, RecordSampler, DataSource, Trait, TRecord
from matchescu.typing._references import (
    EntityReferenceIdentifier,
    EntityReference,
    EntityReferenceIdFactory,
    EntityProfile,
    RecordAdapter,
)


__all__ = [
    "DataSource",
    "EntityReference",
    "EntityReferenceIdentifier",
    "EntityReferenceIdFactory",
    "EntityProfile",
    "Record",
    "RecordSampler",
    "RecordAdapter",
    "Trait",
    "TRecord",
]
